@@ -35,6 +35,7 @@
 #include "colors.h"
 #include "media.h"
 #include "amfwriter.h"
+#include "utilities/compat.h"
 
 struct writebuf
 {
@@ -42,8 +43,6 @@ struct writebuf
   int len;
 };
 
-char showcolor=1;
-
 void b_read(int sock, void* buf, size_t len)
 {
   while(len>0)
@@ -67,6 +66,7 @@ size_t writehttp(char* ptr, size_t size, size_t nmemb, void* x)
 }
 
 CURL* curl=0;
+const char* cookiefile="";
 char* http_get(const char* url, const char* post)
 {
   if(!curl){curl=curl_easy_init();}
@@ -76,7 +76,8 @@ char* http_get(const char* url, const char* post)
   curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, writehttp);
   curl_easy_setopt(curl, CURLOPT_WRITEDATA, &writebuf);
   curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L);
-  curl_easy_setopt(curl, CURLOPT_COOKIEFILE, "");
+  curl_easy_setopt(curl, CURLOPT_COOKIEFILE, cookiefile);
+  curl_easy_setopt(curl, CURLOPT_COOKIEJAR, cookiefile);
   curl_easy_setopt(curl, CURLOPT_USERAGENT, "Mozilla Firefox");
   if(post){curl_easy_setopt(curl, CURLOPT_POSTFIELDS, post);}
   char err[CURL_ERROR_SIZE];
@@ -268,6 +269,9 @@ void usage(const char* me)
          "-u, --user <user>    Username of tinychat account to use.\n"
          "-p, --pass <pass>    Password of tinychat account to use.\n"
          "-c, --color <value>  Color to use in chat.\n"
+         "    --hexcolors      Print hex colors instead of ANSI color codes.\n"
+         "    --cookies <file> File to store cookies in (not having to solve\n"
+         "                       the captchas every time)\n"
 #ifdef RTMP_DEBUG
          "    --rtmplog <file> Write RTMP input to file.\n"
 #endif
@@ -308,9 +312,15 @@ int main(int argc, char** argv)
       ++i;
       currentcolor=atoi(argv[i]);
     }
+    else if(!strcmp(argv[i], "--cookies"))
+    {
+      ++i;
+      cookiefile=argv[i];
+    }
 #ifdef RTMP_DEBUG
     else if(!strcmp(argv[i], "--rtmplog")){++i; rtmplog=open(argv[i], O_WRONLY|O_CREAT|O_TRUNC, 0600); if(rtmplog<0){perror("rtmplog: open");}}
 #endif
+    else if(!strcmp(argv[i], "--hexcolors")){hexcolors=1;}
     else if(!channel){channel=argv[i];}
     else if(!nickname){nickname=strdup(argv[i]);}
     else if(!password){password=argv[i];}
@@ -519,9 +529,9 @@ int main(int argc, char** argv)
             if(!strcmp(&buf[7], "off")){showcolor=0; continue;}
             if(!strcmp(&buf[7], "on")){showcolor=1; continue;}
             currentcolor=atoi(&buf[7]);
-            printf("\x1b[%smChanged color\x1b[0m\n", termcolors[currentcolor%COLORCOUNT]);
+            printf("%sChanged color%s\n", color_start(colors[currentcolor%COLORCOUNT]), color_end());
           }else{ // No color specified, state our current color
-            printf("\x1b[%smCurrent color: %i\x1b[0m\n", termcolors[currentcolor%COLORCOUNT], currentcolor%COLORCOUNT);
+            printf("%sCurrent color: %i%s\n", color_start(colors[currentcolor%COLORCOUNT]), currentcolor%COLORCOUNT, color_end());
           }
           fflush(stdout);
           continue;
@@ -531,7 +541,7 @@ int main(int argc, char** argv)
           int i;
           for(i=0; i<COLORCOUNT; ++i)
           {
-            printf("\x1b[%smColor %i\x1b[0m\n", termcolors[i], i);
+            printf("%sColor %i%s\n", color_start(colors[i]), i, color_end());
           }
           fflush(stdout);
           continue;
@@ -753,10 +763,22 @@ int main(int argc, char** argv)
     {
       size_t len;
       char* msg=fromnumlist(amfin->items[3].string.string, &len);
-      const char* color=(showcolor?resolvecolor(amfin->items[4].string.string):"0");
-      printf("%s \x1b[%sm%s: ", timestamp(), color, amfin->items[5].string.string);
-      fwrite(msg, len, 1, stdout);
-      printf("\x1b[0m\n");
+      const char* color=color_start(amfin->items[4].string.string);
+      char* line=msg;
+      while(line)
+      {
+        // Handle multi-line messages
+        char* nextline=0;
+        unsigned int linelen;
+        for(linelen=0; &line[linelen]<&msg[len]; ++linelen)
+        {
+          if(line[linelen]=='\r' || line[linelen]=='\n'){nextline=&line[linelen+1]; break;}
+        }
+        printf("%s %s%s: ", timestamp(), color, amfin->items[5].string.string);
+        fwrite(line, linelen, 1, stdout);
+        printf("%s\n", color_end());
+        line=nextline;
+      }
       char* response=0;
       if(len==18 && !strncmp(msg, "/userinfo $request", 18))
       {
