@@ -54,6 +54,7 @@
 #include "logging.h"
 #include "../stringutils.h"
 #include "../compat.h"
+#include "inputhistory.h"
 
 struct viddata
 {
@@ -116,9 +117,10 @@ void updatescaling(struct viddata* data, unsigned int width, unsigned int height
   {
     GdkPixbuf* pixbuf=gtk_image_get_pixbuf(GTK_IMAGE(cams[i].cam));
     if(!pixbuf){continue;}
+    GdkPixbuf* old=pixbuf;
     pixbuf=gdk_pixbuf_scale_simple(pixbuf, data->scalewidth, data->scaleheight, GDK_INTERP_BILINEAR);
     gtk_image_set_from_pixbuf(GTK_IMAGE(cams[i].cam), pixbuf);
-// TODO: figure out/fix the "static" noise that seems to happen here
+    g_object_unref(old);
   }
 }
 
@@ -142,6 +144,7 @@ void printchat(const char* text, const char* pm)
   gtk_text_buffer_get_end_iter(buffer, &end);
   gtk_text_buffer_insert(buffer, &end, "\n", -1);
   gtk_text_buffer_insert(buffer, &end, text, -1);
+  buffer_updatesize(buffer);
   if(bottom){autoscroll_after(scroll);}
 }
 
@@ -166,16 +169,38 @@ void printchat_color(const char* text, const char* color, unsigned int offset, c
   gtk_text_buffer_insert(buffer, &end, "\n", -1);
   int startnum=gtk_text_iter_get_offset(&end);
   gtk_text_buffer_insert(buffer, &end, text, -1);
+  GtkTextIter start;
   // Set color if there was one
   if(color)
   {
-    GtkTextIter start;
+    GtkTextTagTable* table=gtk_text_buffer_get_tag_table(buffer);
+    if(!gtk_text_tag_table_lookup(table, color)) // Color isn't defined, define it
+    {
+      char colorbuf[8];
+      memcpy(colorbuf, color, 7);
+      colorbuf[7]=0;
+      while(strchr(colorbuf,',')){memmove(&colorbuf[2], &colorbuf[1], 5); colorbuf[1]='0';}
+      gtk_text_buffer_create_tag(buffer, color, "foreground", colorbuf, (char*)0);
+    }
     gtk_text_buffer_get_iter_at_offset(buffer, &start, startnum+offset);
     gtk_text_buffer_apply_tag_by_name(buffer, color, &start, &end);
   }
+  if(offset==8) // Chat message, has timestamp and nickname, turn them gray and bold
+  {
+    gtk_text_buffer_get_iter_at_offset(buffer, &start, startnum);
+    gtk_text_buffer_get_iter_at_offset(buffer, &end, startnum+offset);
+    gtk_text_buffer_apply_tag_by_name(buffer, "timestamp", &start, &end);
+    unsigned int nicklen=strchr(&text[offset], ' ')-text;
+    gtk_text_buffer_get_iter_at_offset(buffer, &start, startnum+offset);
+    gtk_text_buffer_get_iter_at_offset(buffer, &end, startnum+nicklen);
+    gtk_text_buffer_apply_tag_by_name(buffer, "nickname", &start, &end);
+  }
+  buffer_updatesize(buffer);
   if(bottom){autoscroll_after(scroll);}
 }
 
+void freebuffer(guchar* pixels, gpointer data){free(pixels);}
+
 char buf[1024];
 gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer datap)
 {
@@ -244,15 +269,17 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
     printchat(buf, 0);
     return 1;
   }
-  // Remove escape codes and pick up the text color while we're at it
   char* color=0;
-  char* esc;
-  char* escend;
-  while((esc=strchr(buf, '\x1b'))&&(escend=strchr(esc, 'm')))
+  // Handle color at start of line (/color and /colors)
+  if(buf[0]=='(' && buf[1]=='#')
   {
-    escend[0]=0;
-    if(!color && strcmp(&esc[1], "[0")){color=strdup(&esc[1]);}
-    memmove(esc, &escend[1], strlen(&escend[1])+1);
+    char* end=strchr(buf, ')');
+    if(end)
+    {
+      end[0]=0;
+      color=strdup(&buf[1]);
+      memmove(buf, &end[1], strlen(&end[1])+1);
+    }
   }
   char* space=strchr(buf, ' ');
   // Timestamped events
@@ -260,6 +287,16 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
   {
     char* pm=0;
     char* nick=&buf[8];
+    if(nick[0]=='(' && nick[1]=='#') // Message color
+    {
+      char* end=strchr(nick, ')');
+      if(end)
+      {
+        end[0]=0;
+        color=strdup(&nick[1]);
+        memmove(nick, &end[1], strlen(&end[1])+1);
+      }
+    }
     space=strchr(nick, ' ');
     if(!space){return 1;}
     if(space[-1]==':')
@@ -282,6 +319,9 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
       {
 #ifndef _WIN32
         if(!fork())
+#else
+        char* spacetmp=space;
+        space=strdup(space);
 #endif
         {
 // TODO: store the PID and make sure it's dead before starting a new video? and upon /mbc?
@@ -310,6 +350,8 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
           }
 #ifdef _WIN32
           w32_runcmd(cmd);
+          free(space);
+          space=spacetmp;
 #else
           execlp("sh", "sh", "-c", cmd, (char*)0);
           _exit(0);
@@ -560,17 +602,17 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
 
   // Scale and convert to RGB24 format
   unsigned int bufsize=avpicture_get_size(PIX_FMT_RGB24, scalewidth, scaleheight);
-  unsigned char buf[bufsize];
+  unsigned char* buf=malloc(bufsize);
   cam->dstframe->data[0]=buf;
   cam->dstframe->linesize[0]=scalewidth*3;
   struct SwsContext* swsctx=sws_getContext(cam->frame->width, cam->frame->height, cam->frame->format, scalewidth, scaleheight, AV_PIX_FMT_RGB24, 0, 0, 0, 0);
   sws_scale(swsctx, (const uint8_t*const*)cam->frame->data, cam->frame->linesize, 0, cam->frame->height, cam->dstframe->data, cam->dstframe->linesize);
   sws_freeContext(swsctx);
 
-  GdkPixbuf* gdkframe=gdk_pixbuf_new_from_data(cam->dstframe->data[0], GDK_COLORSPACE_RGB, 0, 8, scalewidth, scaleheight, cam->dstframe->linesize[0], 0, 0);
+  GdkPixbuf* oldpixbuf=gtk_image_get_pixbuf(GTK_IMAGE(cam->cam));
+  GdkPixbuf* gdkframe=gdk_pixbuf_new_from_data(cam->dstframe->data[0], GDK_COLORSPACE_RGB, 0, 8, scalewidth, scaleheight, cam->dstframe->linesize[0], freebuffer, 0);
   gtk_image_set_from_pixbuf(GTK_IMAGE(cam->cam), gdkframe);
-  // Make sure it gets redrawn in time
-  gdk_window_process_updates(gtk_widget_get_window(cam->cam), 1);
+  g_object_unref(oldpixbuf);
 
   return 1;
 }
@@ -645,7 +687,11 @@ void handleresizepane(GObject* obj, GParamSpec* spec, struct viddata* data)
 
 gboolean inputkeys(GtkWidget* widget, GdkEventKey* event, void* data)
 {
-  if(event->keyval==GDK_KEY_Up || event->keyval==GDK_KEY_Down){return 1;}
+  if(event->keyval==GDK_KEY_Up || event->keyval==GDK_KEY_Down)
+  {
+    inputhistory(GTK_ENTRY(widget), event);
+    return 1;
+  }
   if(event->keyval==GDK_KEY_Tab)
   {
     // Tab completion
@@ -695,9 +741,10 @@ gboolean inputkeys(GtkWidget* widget, GdkEventKey* event, void* data)
   return 0;
 }
 
-void sendmessage(GtkEntry* entry, struct viddata* data)
+void sendmessage(GtkEntry* entry, void* data)
 {
   const char* msg=gtk_entry_get_text(entry);
+  inputhistory_add(msg);
   char* pm=0;
   if(!strncmp(msg, "/pm ", 4))
   {
@@ -814,8 +861,8 @@ void startsession(GtkButton* button, void* x)
   const char* acc_user=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "acc_username")));
   const char* acc_pass=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "acc_password")));
 #ifdef _WIN32
-  char cmd[strlen("./tc_client -u    0")+strlen(acc_user)+strlen(channel)+strlen(nick)+strlen(chanpass)];
-  strcpy(cmd, "./tc_client ");
+  char cmd[strlen("./tc_client --hexcolors -u    0")+strlen(acc_user)+strlen(channel)+strlen(nick)+strlen(chanpass)];
+  strcpy(cmd, "./tc_client --hexcolors ");
   if(acc_user[0])
   {
     strcat(cmd, "-u ");
@@ -841,9 +888,9 @@ void startsession(GtkButton* button, void* x)
     dup2(tc_client_in[0], 0);
     if(acc_user[0])
     {
-      execlp(TC_CLIENT, TC_CLIENT, "-u", acc_user, channel, nick, chanpass, (char*)0);
+      execlp(TC_CLIENT, TC_CLIENT, "--hexcolors", "-u", acc_user, channel, nick, chanpass, (char*)0);
     }else{
-      execlp(TC_CLIENT, TC_CLIENT, channel, nick, chanpass, (char*)0);
+      execlp(TC_CLIENT, TC_CLIENT, "--hexcolors", channel, nick, chanpass, (char*)0);
     }
   }
 #endif
@@ -873,6 +920,7 @@ int main(int argc, char** argv)
 #else
   frombuild=1;
 #endif
+  config_load();
 
 #if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
   #ifdef HAVE_AVRESAMPLE
@@ -968,7 +1016,6 @@ int main(int argc, char** argv)
   g_signal_connect(inputfield, "activate", G_CALLBACK(sendmessage), data);
   g_signal_connect(inputfield, "key-press-event", G_CALLBACK(inputkeys), data);
 
-  config_load();
   // Sound
   GtkWidget* option=GTK_WIDGET(gtk_builder_get_object(gui, "soundradio_cmd"));
   g_signal_connect(option, "toggled", G_CALLBACK(toggle_soundcmd), gui);
